@file:OptIn(DelicateCoroutinesApi::class)

package com.bing89.travebing

import android.Manifest
import android.Manifest.*
import android.annotation.SuppressLint
import android.content.ContentValues
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.location.Criteria
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.provider.MediaStore
import android.util.Base64
import android.util.Log
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.annotation.RequiresPermission
import androidx.appcompat.app.AppCompatActivity
import androidx.camera.core.CameraSelector
import androidx.camera.core.ImageCapture
import androidx.camera.core.ImageCaptureException
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.content.FileProvider
import com.bing89.travebing.R.*
import kotlinx.coroutines.*
import java.io.ByteArrayOutputStream
import java.io.File
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale


class MainActivity : AppCompatActivity() {

    private lateinit var startButton: Button
    private lateinit var previewImageView: ImageView
    private lateinit var gpsInfoTextView: TextView
    private var imageCapture: ImageCapture? = null

    private var isMonitoring = false
    private var photoFilePath: String? = null
    private var locationManager: LocationManager? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(layout.activity_main)

        startButton = findViewById(id.startButton)
        previewImageView = findViewById(id.previewImageView)
        gpsInfoTextView = findViewById(id.gpsInfoTextView)
        startCamera()
        startButton.setOnClickListener {
            if (isMonitoring) {
                stopMonitoring()
            } else {
                startMonitoring()
            }
        }
        checkPermissions()
        getLocation()
        locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
    }

    private fun startMonitoring() {
        isMonitoring = true
        startButton.text = "休息"
//        if(imageCapture == null ){
//            startCamera()
//        }
        takePhoto()

//        val intervalMillis = 60000L // 间隔时间，这里设置为1分钟
//        val handler = Handler(Looper.getMainLooper())
//
//        handler.post(object : Runnable {
//            override fun run() {
//                if (isMonitoring) {
//                    getLocation()
////                    capturePhotoAndUpload()
//                    takePhoto()
//                    handler.postDelayed(this, intervalMillis)
//                }
//            }
//        })
    }

    private fun stopMonitoring() {
        isMonitoring = false
        startButton.text = "出发"
    }
    @RequiresPermission(
        anyOf = [permission.ACCESS_COARSE_LOCATION, permission.ACCESS_FINE_LOCATION],
    )

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == PHOTO_REQUEST_CODE ) {
//            doSomeOperations()
        }
    }
    private fun checkPermissions(): Boolean {
        val requiredPermissions = arrayOf(
            permission.CAMERA,
            permission.WRITE_EXTERNAL_STORAGE,
            permission.ACCESS_FINE_LOCATION
        )

        for (permission in requiredPermissions) {
            if (ActivityCompat.checkSelfPermission(
                    this,
                    permission
                ) != PackageManager.PERMISSION_GRANTED
            ) {
                ActivityCompat.requestPermissions(this, requiredPermissions, PERMISSION_REQUEST_CODE)
                return false
            }
        }

        return true
    }
    //define the listener
    private val locationListener: LocationListener = object : LocationListener {
        override fun onLocationChanged(location: Location) {
            updateGpsInfo(location)
        }
        override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}
        override fun onProviderEnabled(provider: String) {}
        override fun onProviderDisabled(provider: String) {}
    }

    private fun createPhotoFile(): File? {
        val timeStamp = SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault()).format(Date())
        val storageDir = getExternalFilesDir(null)
        return File.createTempFile("JPEG_${timeStamp}_", ".jpg", storageDir)
    }

    private fun getLocation(){
        val locMan = getSystemService(Context.LOCATION_SERVICE) as LocationManager
        val checkCameraPermission = ContextCompat.checkSelfPermission(this, permission.ACCESS_FINE_LOCATION)
        val checkCallPhonePermission =
            ContextCompat.checkSelfPermission(this, permission.ACCESS_COARSE_LOCATION)
        if (checkCallPhonePermission != PackageManager.PERMISSION_GRANTED || checkCameraPermission != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, arrayOf(
                permission.ACCESS_COARSE_LOCATION,
                permission.ACCESS_FINE_LOCATION
            ), 22)
        }
        try {
            // Request location updates
            val providers = locMan.getProviders(false)
            var provider = LocationManager.NETWORK_PROVIDER
            if(providers.size != 0) {
                if (providers.contains(LocationManager.GPS_PROVIDER)) {
                    provider = LocationManager.GPS_PROVIDER
                }else if (providers.contains(LocationManager.PASSIVE_PROVIDER)){
                    provider = LocationManager.PASSIVE_PROVIDER
                }
            }
            Toast.makeText(baseContext, "使用" + provider + "定位", Toast.LENGTH_SHORT).show()
            Log.d(TAG, "==================>>>> get location from gps")
            locMan.requestLocationUpdates(provider, 10000L, 100f, locationListener)
        } catch(ex: SecurityException) {
            Log.d(TAG, "Security Exception, no location available")
        }
    }


    private fun updateGpsInfo(location: Location) {
        val latitude = location.latitude
        val longitude = location.longitude
        val gpsInfo = "Latitude: $latitude, Longitude: $longitude"
        gpsInfoTextView.text = gpsInfo
    }

    private fun uploadPhotoAndGpsInfo(photoFilePath: String, gpsInfo: String) {
        // 在此处实现将照片和GPS信息上传到api.abc.com/report接口的逻辑
        // 可以使用 Retrofit 或其他网络库进行网络请求
        // 将照片转换成 Base64 字符串，然后添加到请求中
        // 请求完成后更新界面上的预览窗口等信息
        GlobalScope.launch(Dispatchers.IO) {
            try {
                val photoFile = File(photoFilePath)
                val photoBytes = ByteArrayOutputStream().use {
                    val inputStream = photoFile.inputStream()
                    inputStream.copyTo(it)
                    it.toByteArray()
                }

                val base64Photo = Base64.encodeToString(photoBytes, Base64.DEFAULT)

                // 在此处添加上传逻辑
                // 使用 Retrofit 或其他网络库发送照片和GPS信息到api.abc.com/report接口
                // 请注意处理网络请求的结果和错误
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    companion object {
        private const val PERMISSION_REQUEST_CODE = 1001
        private const val PHOTO_REQUEST_CODE = 1002
        private const val TAG = "Travebing"
        private const val FILENAME_FORMAT = "yyyy-MM-dd-HH-mm-ss-SSS"
        private const val REQUEST_CODE_PERMISSIONS = 10
    }
    private fun startCamera() {
        val cameraProviderFuture = ProcessCameraProvider.getInstance(this)

        cameraProviderFuture.addListener({
            // Used to bind the lifecycle of cameras to the lifecycle owner
            val cameraProvider: ProcessCameraProvider = cameraProviderFuture.get()

            // Preview
//            val preview = Preview.Builder()
//                .build()
//                .also {
//                    it.setSurfaceProvider(viewBinding.viewFinder.surfaceProvider)
//                }
//            val recorder = Recorder.Builder()
//                .setQualitySelector(QualitySelector.from(Quality.HIGHEST,
//                    FallbackStrategy.higherQualityOrLowerThan(Quality.SD)))
//                .build()
//            videoCapture = VideoCapture.withOutput(recorder)

            imageCapture = ImageCapture.Builder().build()

            /*
            val imageAnalyzer = ImageAnalysis.Builder().build()
                .also {
                    setAnalyzer(
                        cameraExecutor,
                        LuminosityAnalyzer { luma ->        startCamera()
                            Log.d(TAG, "Average luminosity: $luma")
                        }
                    )
                }
            */

            // Select back camera as a default
            val cameraSelector = CameraSelector.DEFAULT_BACK_CAMERA

            try {
                // Unbind use cases before rebinding
                cameraProvider.unbindAll()

                // Bind use cases to camera
                cameraProvider.bindToLifecycle(
                    this, cameraSelector,  imageCapture)

            } catch(exc: Exception) {
                Log.e(TAG, "Use case binding failed", exc)
            }

        }, ContextCompat.getMainExecutor(this))
    }
    private fun takePhoto() {
        // Get a stable reference of the modifiable image capture use case
        val imageCapture = imageCapture ?: return
        Log.d(TAG, "================>>>>> now do capture photo")
        // Create time stamped name and MediaStore entry.
        val name = SimpleDateFormat(FILENAME_FORMAT, Locale.US)
            .format(System.currentTimeMillis())
        val contentValues = ContentValues().apply {
            put(MediaStore.MediaColumns.DISPLAY_NAME, name)
            put(MediaStore.MediaColumns.MIME_TYPE, "image/jpeg")
            put(MediaStore.Images.Media.RELATIVE_PATH, "Pictures/CameraX-Image")
        }

        // Create output options object which contains file + metadata
        val outputOptions = ImageCapture.OutputFileOptions
            .Builder(contentResolver,
                MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
                contentValues)
            .build()

        // Set up image capture listener, which is triggered after photo has
        // been taken
        imageCapture.takePicture(
            outputOptions,
            ContextCompat.getMainExecutor(this),
            object : ImageCapture.OnImageSavedCallback {
                override fun onError(exc: ImageCaptureException) {
                    Log.e(TAG, "+++++++++++Photo capture failed: ${exc.message}", exc)
                }

                override fun
                        onImageSaved(output: ImageCapture.OutputFileResults){
                    previewImageView.setImageURI(output.savedUri)
                    val msg = "--------- Photo capture succeeded: ${output.savedUri}"
                    Toast.makeText(baseContext, msg, Toast.LENGTH_SHORT).show()
                    Log.d(TAG, msg)
                }
            }
        )
    }

}
